;    nunimbr.asm  A boot selector which does not use the bios. This
;		   routine is used to choose a partition in which to
;		   find & execute a boot loader.

;    Copyright (C) 1999-2000  Neil Koozer
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;    Neil Koozer, 907 SE Sharon Ave, Roseburg, OR 97470-3824
;    nkoozer@rosenet.net
;-------------------------------------------------------------

section .text

ORG 0x7e00

code_start		;just a label
load_offset	equ 0x200
ptable		equ 0x7c00+0x1be

 cli
 jmp short Entry

drive	dd 0
Count	db 1		;IDE command slate starts here
lba	dd 0
	db 0x20		;read command

;-------------------------edit these for your system---------
io_ab	dw 0x1f0,0x3f6		;ide0 motherboard primary
io_cd	dw 0x170,0x376		;ide1 motherboard secondary
io_ef	dw 0x6b00,0x6f00	;ide2 Ultra66 primary
io_gh	dw 0x7300,0x7700	;ide3 Ultra66 secondary
Command db 'hda0'		;default partition to boot
;------------------------------------------------------------
 db 0			;string terminator
Entry			;interactive command can overlay here
 sub cx,cx		;code used only once
 mov ds,cx
 mov es,cx
 mov ss,cx
 mov sp,cx
; sti
 mov si,0x7c00
 mov di,0x7e00
 mov cx,0x100
 cld
 rep
 movsw
 jmp start+load_offset

start
iobase		equ start-8	;overlay some variables
ext_start	equ start-4
 mov AH,2		;check for shift, ctrl, or alt
 int 0x16
 and AL,0xF
 jz f1			;use default if none pressed
prompt			;errors jump to command prompt
 call Input
f1
 mov al,[Command+2]
 sub al,'a'
 jc prompt

 mov bl,al
 add bl,bl
 and bx,0xfc
 mov edx,[bx + io_ab]
 mov [iobase],edx

 and al,1		;drive bit
 shl al,4
 or al,0x40		;insert lba bit
 mov [drive+3],al

 mov DX,[iobase+2]
 mov AL,2		;use polling for IDE device
 out DX,AL

 mov DX,[iobase]
 add DX,byte 7
.b1 in AL,DX		;wait till controller ready
 and AL,0x88
 jnz .b1

 sub eax,eax		;load mbr
 call load7c00
 mov ax,[Command+3]	;parse partition number
 sub al,'0'
 je prompt		;hda0 -> new prompt
 jb done		;hda ok
 sub ah,'0'
 jc one_digit
 mov dl,ah		;allow up to 99 partitions per drive
 mov ah,10
 mul ah
 add al,dl
one_digit
 cmp al,4
 ja extended
 shl al,4
 sub bx,bx
 mov bl,al
 mov eax,[bx+ptable-8]
 or eax,eax		;0 = non-existant partition
 je prompt
done0
 call load7c00
done
 mov DX,[iobase+2]
 mov AL,0		;restore int mode for IDE
 out DX,AL
 jmp 0x7c00		;execute the thing we loaded

extended
 mov cx,4		;look for an extended partition
 mov bx,ptable+4
.b1
 cmp [bx],byte 5
 je go_ext
 cmp [bx],byte 0xf
 je go_ext
 cmp [bx],byte 0x85
 je go_ext
 add bx,16
 loop .b1
error
 jmp prompt		;no extended partition, start over
go_ext
 sub al,4		;fix the count
 mov cl,al
 mov eax,[bx+4]
 mov [ext_start],eax	;start sector of extended partition
 sub ebx,ebx
.b1
 cmp eax,ebx
 jb error		;reject reading beyond last logical partition
 mov ebx,eax
 call load7c00
 mov eax,[ptable+24]
 add eax,[ext_start]
 loop .b1
 mov eax,ebx
 add eax,[ptable+8]
 jmp short done0

Input
 mov BX,Command+2
 call PrintThis
 db 10,13,'Partition to boot: hd',0
.more
 mov AH,0
 int 0x16		;get char
 mov [BX],AL
 cmp AL,0x0D
 je .out
 cmp AL,8
 jne .f1
 cmp BX,Command+2
 je .more
 call PrintThis
 db 8,' ',8,0
 dec BX
 jmp short .more
.f1
 cmp AL,0x20
 jb .more
 cmp AL,0x7e
 ja .more
 call PrintChar
 inc BX
 jmp short .more
.out
 call PrintThis
 db 10,13,0
 sub AL,AL
 mov [BX],AL
 ret

PrintChar
 push BX
 sub BH,BH		;video page 0
 mov AH,0xE
 int 0x10
 pop BX
 ret

PrintThis
 pop SI			;address of string
.more
 mov AL,[CS:SI]
 inc SI
 or AL,AL		;end of asciiz string?
 je .out
 call PrintChar
 jmp short .more
.out
 jmp SI

load7c00
 mov di,0x7c00
ReadSector
 or eax,[drive]
 mov [lba],eax
 push cx
 mov SI,Count		;start of six-byte IDE command slate
 mov cx,6		;one command to read a multi-sector ext2 block
 mov DX,[iobase]
 inc DX
@@b3			;load 6 bytes into controller
 inc DX
 outsb
 loop @@b3
;data
 push CX
 mov CX,300		;wait >= 400 ns
.b2 loop .b2
@@b2			;poll for a drq
 in AL,DX
 and AL,88h
 jnz .f2
 jmp error		;error, start over
.f2
 cmp AL,8
 jnz @@b2
 push DX
 mov DX,[iobase]	;get data
 mov cx,0x100		;1 sector = 0x100 words
 rep insw
 pop DX
 pop CX
 pop cx
 ret

TIMES 510-($-$$) DB 0
 dw 0xaa55
End

